#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Утилита для разбора XML с единым реестром запрещённых сайтов
Реестр преобразуется в простой текстовый файл, подходящий для Carbon Reductor
"""

import os
import re
import sys
from urllib import splitport
from urlparse import urlparse
import xml.etree.ElementTree as etree
from lower_host_lib import is_only_url

__author__ = 'oleg'

HTTP_LIST, HTTPS_LIST, HTTP_PORTS, HTTPS_PORTS, IP_BLOCK = [], [], [], [], []
IP_HTTPS_LIST, IP_FTP_LIST, OTHER_URLS, FULLWHITELIST = [], [], [], []
HTTPS_URLS = []
SCHEMES_REST = ('http', 'https', 'ftp',)

GLOBAL_LISTS = (
    (HTTP_LIST, "urls to load", "add_url"),
    (HTTPS_LIST, "https domains to resolv", "need_resolv"),
    (HTTPS_URLS, "https urls now for test only", "add_url_https"),
    (IP_BLOCK, "ip to full block", "add_ip_block"),
    (IP_HTTPS_LIST, "ip to block on https", "add_ip_https"),
    (HTTP_PORTS, "http ports", "add_http_port"),
    (HTTPS_PORTS, "https ports", "add_https_port"),
    (IP_FTP_LIST, "ftp ip", "add_ip_ftp"),
    (OTHER_URLS, "other urls by ip + port", "add_ip_port"),
)


def filter_by_scheme(url_list, scheme):
    """ Даёт список URL по нужному протоколу, проще чем анализ по одному """
    return [url for url in url_list if urlparse(url).scheme == scheme]


def remove_url_default_port(url):
    """ избавляемся от стандартных портов HTTP/HTTPS """
    port = None
    if url.startswith('https://'):
        port = '443'
    elif url.startswith('http://'):
        port = '80'
    else:
        return url
    return pick_port(url) == port and url.replace(':' + port, '', 1) or url


def pick_port(url):
    """ Извлечение порта из URL, если нет - вернёт None """
    return splitport(urlparse(url).netloc)[1]


def process_http(urls_http):
    """ Логика с самым основным - http url'ами """
    urls_http = [remove_url_default_port(url) for url in urls_http]
    whitelisted = False
    for url in list(urls_http):
        if url.rstrip('/') in FULLWHITELIST or url in FULLWHITELIST:
            urls_http.remove(url)
            whitelisted = True
            continue
        if is_only_url(url):
            urls_http = [url.rstrip('/')]
            break
    return urls_http, whitelisted


def process_https(urls):
    """ избавляемся от :443 например """
    return [remove_url_default_port(url) for url in filter_by_scheme(urls, 'https')]


def process_rest(ip_list, port_list_rest):
    """ Обработка других URL, не столь критично, как http, но тоже важно """
    add_crosslist_values(ip_list, port_list_rest, OTHER_URLS)


def add_crosslist_values(source_list1, source_list2, target):
    """ добавляет комбинации содержимого двух списков в третий """
    for item1 in source_list1:
        for item2 in source_list2:
            target.append(item1 + " " + item2)


def process_elem(elem):
    """ Обрабатывает одну запись в реестре. Распихивает её данные по глобальным
    спискам, предварительно сформировав свои """
    ip_regex = r"^([0-9]{1,3}\.){3}[0-9]{1,3}$"
    domain_list = [unicode(d.text).strip() for d in elem.findall('domain')]
    # для того чтобы не добавлять ip адреса в список доменов для резолва и DNS блока
    have_domains = bool(domain_list)
    domain_list = [d for d in domain_list if not re.match(ip_regex, d)]
    ip_list = [ip.text for ip in elem.findall('ip') + elem.findall('ipSubnet')]

    urls = [unicode(url.text).strip() for url in elem.findall('url')]
    urls_http = filter_by_scheme(urls, 'http')
    urls_https = process_https(urls)
    urls_ftp = filter_by_scheme(urls, 'ftp')
    urls_rest = [u for u in urls if urlparse(u).scheme not in SCHEMES_REST]
    HTTPS_URLS.extend(urls_https)
# pylint: disable=W0141
    HTTP_PORTS.extend([port for port in map(pick_port, urls_http) if port])
    port_list_https = [port for port in map(
        pick_port, urls_https) if port and port != '443']
    port_list_rest = [port for port in map(pick_port, urls_rest) if port]
# pylint: enable=W0141
    urls_http, whitelisted = process_http(urls_http)
    process_rest(ip_list, port_list_rest)
    HTTPS_PORTS.extend(port_list_https)

    # пытаемся избежать полной блокировки ip создав urls_http из других источников
    if urls_https:
        HTTPS_LIST.extend(domain_list)
        urls_http.extend([url.replace('https', 'http', 1)
                          for url in urls_https])
        urls_http, whitelisted = process_http(urls_http)
        IP_HTTPS_LIST.extend(ip_list)
        add_crosslist_values(ip_list, port_list_https, OTHER_URLS)
    if not urls_https and have_domains and not urls_http:
        IP_HTTPS_LIST.extend(ip_list)
    if not urls_https and os.environ.get('BLOCK_HTTPS_IP_EVEN_IF_HTTP') == '1':
        IP_HTTPS_LIST.extend(ip_list)

    if not urls_http and domain_list and not whitelisted:
        urls_http = ["http://" + d for d in domain_list]
        IP_HTTPS_LIST.extend(ip_list)
        HTTPS_LIST.extend(domain_list)

    if urls_ftp:
        IP_FTP_LIST.extend(ip_list)

    if not urls_http:
        IP_BLOCK.extend(ip_list)

    HTTP_LIST.extend(urls_http)


def file_to_list(filename):
    """ Возвращает список строк из файла """
    new_list = []
    if os.path.isfile(filename):
        with open(filename, 'r') as url_file:
            return [unicode(s, 'utf-8').strip() for s in url_file.xreadlines()]
    return new_list


def print_list(list_structure):
    """ Выводит список на экран с нужными префиксами и комментом-описанием """
    item_list, describe, prefix = list_structure
    print "#", describe
    for item in sorted(set(item_list)):
        print prefix, item.encode('utf-8')


def print_lists():
    """ Печатает все списки на экран """
    for list_structure in GLOBAL_LISTS:
        print_list(list_structure)


def process_dump_xml(dump):
    """ Обрабатывает содержимое dump.xml """
    for entry in etree.parse(dump).getroot():
        process_elem(entry)

if __name__ == '__main__':
    FULLWHITELIST = file_to_list('/usr/local/Reductor/lists/our.fullwhitelist')
    DUMP = "/usr/local/Reductor/reductor_container/gost-ssl/php/dump.xml"
    # DUMP = "reductor_container/gost-ssl/php/dump.xml"
    if len(sys.argv) == 2:
        DUMP = sys.argv[1]
    process_dump_xml(DUMP)
    print_lists()
