#!/usr/bin/python
# coding=utf8

# -*- author: oleg -*-

import suds, base64, time, zipfile, os, sys

if os.environ.get('SUDS_NO_CACHE'):
    client = suds.client.Client("http://vigruzki.rkn.gov.ru/services/OperatorRequest/?wsdl", cache=None)
else:
    client = suds.client.Client("http://vigruzki.rkn.gov.ru/services/OperatorRequest/?wsdl")

chrootdir = "/usr/local/Reductor/reductor_container/gost-ssl/php/"
zip = chrootdir + "register.zip"
request = "request.xml"
output = "dump.xml"
main_download = True
if len(sys.argv) == 2 and sys.argv[1] == '--second':
    main_download = False
if not main_download:
    request = "request_second.xml"
    print("Performing second request")
signature = request + ".sign"

def u_print(str):
    print(str.encode('utf8'))


def prepare():
    if not main_download:
        return
    u_print(u'Удаляем предыдущий архив.')
    if os.path.exists(zip):
        os.unlink(zip)


def get_code():
    u_print(u'Запрашиваем идентификатор запроса:')
    request = sendRequest()
    if 'code' in request:
        u_print(u'Идентификатор:' + str(request['code']))
        return request['code']
    u_print(u'Не удалось получить идентификатор запроса')
    if 'resultComment' in request:
        print request['resultComment']
    print(request)
    exit(1)


def file_data_base64(filename):
    file = open(filename, "rb")
    data = file.read()
    file.close()
    return base64.b64encode(data)


def result2dict(result):
    return dict(((k, v.encode('utf-8')) if isinstance(v, suds.sax.text.Text) else (k, v)) for (k, v) in result)


def sendRequest():
    u_print(u'Отправка запроса')
    xml = file_data_base64(chrootdir + request)
    sign = file_data_base64(chrootdir + request + ".sign")
    result = client.service.sendRequest(xml, sign, "2.2")
    if not result:
        u_print(u'Отсутствует ответ от Роскомнадзора')
        exit(2)
    return result2dict(result)


def getResult(code):
    u_print(u' Запрос архива (' + str(code) + ')')
    result = client.service.getResult(code)
    return result2dict(result)


def download(code, sleep_time=30):
    u_print(u'Скачивание архива:')
    result = getResult(code)
    line = u''
    while result['resultCode'] <= 0:
        if sleep_time > 150:
            u_print(u'Превышено количество запросов к серверу Роскомнадзора')
            exit(3)
        u_print(u'  пауза (' + str(sleep_time) + u') секунд..')
        time.sleep(sleep_time)
        sleep_time += 10
        result = getResult(code)
        if 'resultComment' in result:
            line = u'  Комментарий: ' + result['resultComment'].decode('utf-8')
        if 'inn' in result:
            line = line + u'. Инн: ' + result['inn'].decode('utf-8')
        if 'resultCode' in result:
            u_print(line + ' (' + str(result['resultCode']) + ')')
            if result['resultCode'] < 0:
                u_print(u' Запрос списков завершён с ошибкой')
                exit(4)
        if 'registerZipArchive' in result:
            u_print(u' zip-архив присутствует в ответе')
            continue
    if result and result['resultCode'] == 1:
        u_print(u' Запрос списков завершён успешно')
    return result


def unpack(result):
    u_print(u'Распаковка архива')
    if not 'registerZipArchive' in result:
        u_print(u'В ответе РосКомНадзора отсутствует архив.')
        exit(5)
    os.chdir(chrootdir)
    if not main_download:
        os.chdir("/tmp/")
    with open(zip, 'w') as f:
        f.write(base64.b64decode(result['registerZipArchive']))
    if zipfile.ZipFile(zip).extract('dump.xml'):
        u_print(u'Список запрещённых сайтов успешно извлечён')


def write_code(code):
    with open(chrootdir + 'dump.xml.code', 'w') as f:
        f.write(code + '\n')


def main():
    prepare()
    code = get_code()
    result = download(code)
    unpack(result)
    write_code(code)


if __name__ == '__main__':
    main()
